/*--------------------------------------------------------------------------*/
/*  PLOT_ANSI: package for simple 2d X11 graphics                           */
/*                                                                          */
/*  (c) by A. Schmidt (1990-2004)                                           */
/*                                                                          */
/*     This program is free software; you can redistribute it and/or modify */
/*     it under the terms of the GNU General Public License as published by */
/*     the Free Software Foundation; either version 2 of the License, or    */
/*     any later version.                                                   */
/*                                                                          */
/*     This program is distributed in the hope that it will be useful,      */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/*     GNU General Public License for more details.                         */
/*--------------------------------------------------------------------------*/

#include <stdio.h>

#include <fcntl.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define single 

#include "PLOTansi.h"


/**********************************************************************
  WindowHandle *PLOT_create_window(name,geometry)
       char *name,*geometry;

  create a new current PLOT window with given name and default geometry
***********************************************************************/

PLOT_INFO *PLOT_CREATE_WINDOW(name,geometry)
     char **name,**geometry;
{
  extern PLOT_INFO *PLOT_create_window();

  return(PLOT_create_window(*name,*geometry));
}



/**********************************************************************
  void PLOT_destroy_window()

  destroy a PLOT window
***********************************************************************/

void PLOT_DESTROY_WINDOW(info)
     PLOT_INFO **info;
{
  extern void PLOT_destroy_window();
  PLOT_destroy_window(*info);
}



/**********************************************************************
  PLOT_INFO *PLOT_get_std_window()

  return PLOT_INFO of current PLOT window
***********************************************************************/

/*PLOT_INFO *PLOT_GET_STD_WINDOW()*/
int PLOT_GET_STD_WINDOW()
{
  extern PLOT_INFO *PLOT_get_std_window();
  int i;
  i = (int)PLOT_get_std_window();
  return(i);
}



/**********************************************************************
  PLOT_INFO *PLOT_set_std_window(new_window)
       PLOT_INFO *new_window;

  set current PLOT window to new_window
  return PLOT_INFO of previously current PLOT window
***********************************************************************/

PLOT_INFO *PLOT_SET_STD_WINDOW(new_window)
     int *new_window;
/*     PLOT_INFO **new_window;*/
{
  extern PLOT_INFO *PLOT_set_std_window();
  PLOT_INFO *info;

  info = (PLOT_INFO *)*new_window;

  return(PLOT_set_std_window(info));
}





/**********************************************************************
  PLOT_expose(window)
       Window window;

  recalculate window size after an expose
***********************************************************************/

PLOT_EXPOSE(window)
     Window *window;
{
  PLOT_expose(*window);
}




/**********************************************************************
  PLOT_title(title)
       char *title

  set title of current PLOT window
***********************************************************************/

PLOT_TITLE(title)
     char **title;
{
  PLOT_title(*title);
}




/**********************************************************************
  PLOT_alarm(sec)
       unsigned sec;

  start the alarm window handler with interval of <sec> seconds
***********************************************************************/

PLOT_ALARM(sec)
     unsigned *sec;
{
  PLOT_alarm(*sec);
}




/************************ icolor ************/

PLOT_SET_ICOLOR(icolor)
    int *icolor;
{
  PLOT_set_icolor(*icolor);
}

PLOT_GET_ICOLOR(icolor)
    int *icolor;
{
  PLOT_get_icolor(*icolor);
}


/************************ background color ************/

PLOT_SET_BACKGROUNDCOLOR(color)
    float color[3];
{
  PLOT_set_backgroundcolor(color);
}

PLOT_GET_BACKGROUND_COLOR(color)
    float color[3];
{
  PLOT_get_backgroundcolor(color);
}

/************************ line color ************/

PLOT_SET_LINECOLOR(color)
     float color[3];
{
  PLOT_set_linecolor(color);
}

PLOT_GET_LINECOLOR(color)
     float color[3];
{
  PLOT_get_linecolor(color);
}

/************************ patch color ************/

PLOT_SET_PATCHCOLOR(color)
     float color[3];
{
  PLOT_set_patchcolor(color);
}

PLOT_GET_PATCHCOLOR(color)
     float color[3];
{
  PLOT_get_patchcolor(color);
}

/************************ text color ************/

PLOT_SET_TEXTCOLOR(color)
     float color[3];
{
  PLOT_set_textcolor(color);
}

PLOT_GET_TEXTCOLOR(color)
     float color[3];
{
  PLOT_get_textcolor(color);
}




/******************************************************************
  Flush
******************************************************************/

PLOT_FLUSH()
{
  PLOT_flush();
}




/******************************************************************
  Clear
******************************************************************/

PLOT_CLEAR()
{
  PLOT_clear();
}




/******************************************************************
  Transformations
******************************************************************/

PLOT_SET_TRANSFORM2D(mtype, mat)
     int *mtype;
     float mat[3][3];
{
  PLOT_set_transform2d(*mtype, mat);
}



PLOT_GET_TRANSFORM2D(mtype, mat)
     int *mtype;
     float mat[3][3];
{
  PLOT_get_transform2d(*mtype, mat);
}


PLOT_WORLD2D(float *xmin, float *xmax, float *ymin, float *ymax)
{
  PLOT_world2d(*xmin, *xmax, *ymin, *ymax);
}


/******************************************************************
  Move
******************************************************************/

PLOT_MOVE2D(x,y)
     float *x,*y;
{
  PLOT_move2d(*x,*y);
}


PLOT_VMOVE2D(vec)
     float vec[2];
{
  PLOT_vmove2d(vec);
}



/******************************************************************
  Draw
******************************************************************/

PLOT_DRAW2D(x,y)
     float *x,*y;
{
  PLOT_draw2d(*x,*y);
}


PLOT_VDRAW2D(vec)
     float vec[2];
{
  PLOT_vdraw2d(vec);
}


/******************************************************************
  Line
******************************************************************/

PLOT_LINE2D(x1,y1, x2,y2)
     float *x1,*y1,*x2,*y2;
{
  PLOT_line2d(*x1,*y1, *x2,*y2);
}


PLOT_VLINE2D(vec1,vec2)
     float vec1[2],vec2[2];
{
  PLOT_vline2d(vec1,vec2);
}


/******************************************************************
  triangle
******************************************************************/

PLOT_TRIANGLE2D(x1,y1, x2,y2, x3,y3)
     float *x1,*y1,*x2,*y2,*x3,*y3;
{
  PLOT_triangle2d(*x1,*y1, *x2,*y2, *x3,*y3);
}


PLOT_VTRIANGLE2D(vec1,vec2,vec3)
     float vec1[2],vec2[2],vec3[2];
{
  PLOT_vtriangle2d(vec1,vec2,vec3);
}



PLOT_TRIANGLERGB2D(x1,y1, x2,y2, x3,y3, rgb1,rgb2,rgb3)
     float *x1,*y1,*x2,*y2,*x3,*y3;
     float rgb1[3],rgb2[3],rgb3[3];
{
  PLOT_triangleRGB2d(*x1,*y1, *x2,*y2, *x3,*y3, rgb1,rgb2,rgb3);
}


PLOT_VTRIANGLERGB2D(vec1,vec2,vec3, rgb1,rgb2,rgb3)
     float vec1[2],vec2[2],vec3[2];
     float rgb1[3],rgb2[3],rgb3[3];
{
  PLOT_vtriangleRGB2d(vec1,vec2,vec3, rgb1,rgb2,rgb3);
}



/******************************************************************
  text
******************************************************************/


PLOT_TEXT2D(x,y,string)
     float *x,*y;
     char **string;
{
  PLOT_text2d(*x,*y,*string);
}



PLOT_VTEXT2D(vec,string)
     float vec[2];
     char **string;
{
  PLOT_vtext2d(vec,*string);
}


