/*--------------------------------------------------------------------------*/
/*  PLOT_ANSI: package for simple 2d X11 graphics                           */
/*                                                                          */
/*  (c) by A. Schmidt (1990-2004)                                           */
/*                                                                          */
/*     This program is free software; you can redistribute it and/or modify */
/*     it under the terms of the GNU General Public License as published by */
/*     the Free Software Foundation; either version 2 of the License, or    */
/*     any later version.                                                   */
/*                                                                          */
/*     This program is distributed in the hope that it will be useful,      */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/*     GNU General Public License for more details.                         */
/*--------------------------------------------------------------------------*/

/******************************************************************
  PLOT LOG package
******************************************************************/

#include <stdio.h>

#include "PLOTansi.h"

extern PLOT_INFO *current_PLOT_info;
#define INFO current_PLOT_info


/******************************************************************
  PLOT_LOG_ITEM *PLOT_LOG_get_item()

  return pointer to free log item, or NULL
******************************************************************/

PLOT_LOG_ITEM *PLOT_LOG_get_item(void)
{
  int i;
  PLOT_LOG *log = INFO->log;
  
  if (log == NULL) {
    if ((log = (PLOT_LOG *)MALLOC(sizeof(PLOT_LOG)))) {
      /* printf("malloc first log %d\n",log); */
      log->used = 0;
      log->flushed = 0;
      log->prev = NULL;
      log->next = NULL;
      INFO->log = log;
      for (i=0;i<PLOT_LOG_SIZE;i++) log->item[i].type = 0;
    }
    else {
      printf("cannot alloc new PLOT_LOG\n");
      return(NULL);
    }
  }

  if (log->used >= PLOT_LOG_SIZE) {
    if ((log = (PLOT_LOG *)MALLOC(sizeof(PLOT_LOG)))) {
      /* printf("malloc next log %d\n",log); */
      log->used = 0;
      log->flushed = 0;
      log->prev = INFO->log;    /* always append new log behind old ones */
      log->next = NULL;
      INFO->log->next = log;
      INFO->log = log;
      for (i=0;i<PLOT_LOG_SIZE;i++) log->item[i].type = 0;
    }
    else {
      printf("cannot alloc new PLOT_LOG\n");
      return(NULL);
    }
  }

  return(&log->item[log->used++]);
}





/******************************************************************
  PLOT_ITEM *next_item()

  increment item pointer
******************************************************************/

static PLOT_LOG *plot_log;
static int plot_log_index;


static PLOT_LOG_ITEM *next_item(void)
{
  if (plot_log_index >= plot_log->used) {
    if ((plot_log = plot_log->next)) {
      /* printf("reach next log %d\n",plot_log); */
      plot_log_index = 0;
      plot_log->flushed = 0;
      if (plot_log->used < 1) {
	return(NULL);
      }
    }
    else {
      return(NULL);
    }
  }
  plot_log->flushed = plot_log_index+1;
  return(&(plot_log->item[plot_log_index++]));


/************
  if (plot_log_index < plot_log->used) {
    plot_log->flushed = plot_log_index+1;
    return(&(plot_log->item[plot_log_index++]));
  }
  else 
    if (plot_log = plot_log->next) {
      plot_log_index = 1;
      if (plot_log->used > 0) {
	plot_log->flushed = 1;
	return (plot_log->item);
      }
      else {
	plot_log->flushed = 0;
      }
    }
  return(NULL);
*************/

}



/******************************************************************
  PLOT_LOG_flush( PLOT_INFO *info );

  old: flush plot log (clear window and display log contents)
  new: flush plot log (only previously non-flushed items)
******************************************************************/

int PLOT_LOG_flushing = 0;


#define SET_COLOR(c)  c.r = item->U.f[0]; \
                      c.g = item->U.f[1]; \
                      c.b = item->U.f[2]

#define SET_POINT(p)  p.x = item->U.f[0] * min_wh; \
                      p.y = height - item->U.f[1] * min_wh; \
                      p.z = item->U.f[2] * min_wh

#define SET_XCOLOR(c)  
#if 1
#define SET_XPOINT(p)  p.x = item->U.f[0] * info->windowWidth; \
                       p.y = (1.0 - item->U.f[1]) * info->windowHeight;
#else
#define SET_XPOINT(p)  p.x = item->U.f[0] * min_wh; \
                       p.y = height - item->U.f[1] * min_wh;
#endif


void PLOT_LOG_flush(PLOT_INFO  *info)
{
  static PLOT_LOG_ITEM *item;
  static XColor xcolor/*, triang_xcolor[4] */;
#define LINE_LEN 100
  static XPoint  line_xpoint[LINE_LEN], triang_xpoint[4], text_xpoint;
  static int line_len = 0;
  static float min_wh, height;
#define STRING_LEN 100
  static char string[STRING_LEN];
  static int i,string_len,clear_window;
  static Display *display;
  static Window window;
  static GC gc;

#ifdef XDIRECT
  static WindowHandle *wh;
  static XdColor color, triang_color[3];
  static XdPoint line_point[2], triang_point[3], text_point;
  static long backgroundPixel;
#else
  static struct {float r,g,b;} color;
#endif


  PLOT_LOG_flushing = 1;

#ifdef XDIRECT
  wh = info->wh;
#endif

  display = info->display;
  window = info->window;
  gc = info->gc;

  /* printf("PLOT_LOG_flush(%d), wh=%d, window=%d \n",info,wh,window);*/

  height = min_wh = info->windowHeight;
  if (min_wh > info->windowWidth)  min_wh = info->windowWidth;
  
  if (min_wh <= 1) {
/*    printf("don't flush a window of size 0\n"); */
    return;
  }


  if (PLOT_redraw) {
    printf("redraw in PLOT_LOG_flush 1\n");
    return;
  }


  /* now, work on info->log ! */

  if ((plot_log = info->log)) {
    for (i=0;i<10000;i++) {
      if (plot_log->prev) {
	plot_log = plot_log->prev;
      }
      else break;
    }
    if (plot_log->prev) {
      printf("PLOT_LOG_flush: endless loop.\n");
      return;
    }

  }
  
  clear_window = 0;
  if (plot_log) {
    if (plot_log->flushed == 0) {
      clear_window = 1;
    }
  }
  if (!plot_log) {
    clear_window = 1;
  }
  
  if (clear_window) {
#ifdef XDIRECT
    if (wh) {
      color.r = info->background_color[0];
      color.g = info->background_color[1];
      color.b = info->background_color[2];
      backgroundPixel = 
	(((int)(255*color.r)) << 16) 
	  |  (((int)(255*color.g)) << 8) 
	    |  (((int)(255*color.b)) );

      XdFillRectangle(wh, 0,0, info->windowWidth, info->windowHeight,
		      backgroundPixel, 0, FillZColor);
    }
    else 
#endif
      {
      if ((info->background_color[0]+info->background_color[1]
	   +info->background_color[2]) >= 1.5) {
	XSetWindowBackground(info->display,info->window,
	          WhitePixel(info->display,DefaultScreen(info->display)));
      }
      else {
	XSetWindowBackground(info->display,info->window,
	          BlackPixel(info->display,DefaultScreen(info->display)));
      }

      XClearWindow(info->display,info->window);
    } 
  }

  plot_log_index = 0;
  while(plot_log) {
    if (plot_log->flushed) {
      if (plot_log->flushed >= plot_log->used) {
	plot_log = plot_log->next;
      }
      else {
	plot_log_index = plot_log->flushed;
	break;
      }
    }
    else break;
  }

  if (plot_log) {
    
#ifdef XDIRECT
    if (wh) {  /* Xdirect display */

      item = next_item();
      while (item) {

	if (PLOT_redraw) {
	  printf("redraw in PLOT_LOG_flush 2\n");
	  return;
	}

	switch (item->type) {

	case PLOT_LOG_color:
	  SET_COLOR(color);
	  XdSetBaseColor(wh, &color);
	  item = next_item();
	  break;

	case PLOT_LOG_move:
	  SET_POINT(line_point[0]);
	  item = next_item();
	  break;

	case PLOT_LOG_draw:
	  SET_POINT(line_point[1]);
	  XdDrawLine(wh, line_point);
	  line_point[0]=line_point[1];
	  item = next_item();
	  break;
	  
	case PLOT_LOG_triang:
	  SET_POINT(triang_point[0]);
	  if (item = next_item()) {
	    SET_POINT(triang_point[1]);
	    if (item = next_item()) {
	      SET_POINT(triang_point[2]);
	      XdFillTriangle(wh, triang_point);
	    }
	  }
	  item = next_item();
	  break;

	case PLOT_LOG_triangRGB:
	  SET_POINT(triang_point[0]);
	  if (item = next_item()) {
	    SET_COLOR(triang_color[0]);
	    if (item = next_item()) {
	      SET_POINT(triang_point[1]);
	      if (item = next_item()) {
		SET_COLOR(triang_color[1]);
		if (item = next_item()) {
		  SET_POINT(triang_point[2]);
		  if (item = next_item()) {
		    SET_COLOR(triang_color[2]);
		    XdFillTriangleRGB(wh, triang_point, triang_color);
		  }
		}
	      }
	    }
	  }
	  item = next_item();
	  break;

	case PLOT_LOG_text:
	  SET_XPOINT(text_xpoint);
	  string_len = 0;
	  while(item = next_item()){
	    if (item->type != PLOT_LOG_string) break;
	    for (i=0;i<PLOT_LOG_charlen; i++) {
	      if (item->U.c[i] == 0) break;
	      if (string_len < (STRING_LEN-1)) {
		string[string_len++] = item->U.c[i];
	      }
	    }
	  }
	  string[string_len] = 0;  /* add a final NUL */
	  
	  XDrawString(display,window,gc,
		      text_xpoint.x,text_xpoint.y,string,string_len);

	  printf(" I'm not able to draw text '%s'\n",string);
	  
	  /* new_item is already done! */
	  break;

	case PLOT_LOG_null:
	  printf("PLOT_LOG_flush: null item\n");
	  break;

	default:
	  printf("PLOT_LOG_flush: unknown item type %d\n",item->type);
	  break;
	}
      }
      XdFlush(wh);

    }
    else 
#endif
{  /* standard X11 display */
    
      line_len = 0;
      item = next_item();
      while (item) {
	
	if (PLOT_redraw) {
	  printf("redraw in PLOT_LOG_flush 2\n");
	  return;
	}

	switch (item->type) {

	case PLOT_LOG_color:
	/*    printf(" C");*/
	  SET_COLOR(color);

#if 1
	  xcolor.red = (255<<8)*color.r;
	  xcolor.green = (255<<8)*color.g;
	  xcolor.blue = (255<<8)*color.b;
	  if (XAllocColor(info->display, 
		  DefaultColormap(info->display,DefaultScreen(info->display)),
			  &xcolor))
	  {
	    XSetForeground(info->display,info->gc, xcolor.pixel);
	  }
	  else
#endif
	    if ((color.r + color.g + color.b) >= 1.5) {
	      XSetForeground(info->display,info->gc,
		   WhitePixel(info->display,DefaultScreen(info->display)));
	    }
	    else {
	      XSetForeground(info->display,info->gc,
		   BlackPixel(info->display,DefaultScreen(info->display)));
	    }

	  item = next_item();
	  break;


	case PLOT_LOG_icolor:
/*	  printf(" c%d ",item->U.i[0]); */

	  if (line_len > 1) {
	    XDrawLines(display, window, gc, 
		       line_xpoint, line_len, CoordModeOrigin);
	  }

	  XSetForeground(info->display,info->gc, item->U.i[0]);
	  item = next_item();
	  break;

	case PLOT_LOG_move:
	/*    printf(" MV");*/
	  if (line_len > 1) {
	    XDrawLines(display, window, gc, 
		       line_xpoint, line_len, CoordModeOrigin);
	  }
	  SET_XPOINT(line_xpoint[0]);
	  line_len = 1;
	  item = next_item();
	  break;

	case PLOT_LOG_draw:
	/*    printf(" DR");*/
	  if (line_len >= LINE_LEN) {
	    XDrawLines(display, window, gc, 
		       line_xpoint, line_len, CoordModeOrigin);
	    line_xpoint[0] = line_xpoint[line_len-1];
	    line_len = 1;
	  }
	  SET_XPOINT(line_xpoint[line_len]);
	  line_len++;

	  item = next_item();
	  break;

	case PLOT_LOG_triang:
	/*    printf(" TR");*/
	  SET_XPOINT(triang_xpoint[0]);
	  if ((item = next_item())) {
	    SET_XPOINT(triang_xpoint[1]);
	    if ((item = next_item())) {
	      SET_XPOINT(triang_xpoint[2]);
	      triang_xpoint[3] = triang_xpoint[0];
/*	      XDrawLines(display, window, gc, 
			 triang_xpoint, 4, CoordModeOrigin); */
	      XFillPolygon(display, window, gc, 
			 triang_xpoint, 4, Convex, CoordModeOrigin);
	    }
	  }
	  item = next_item();
	  break;

	case PLOT_LOG_triangRGB:
	/*    printf(" TC");*/
	  SET_XPOINT(triang_xpoint[0]);
	  if ((item = next_item())) {
	    SET_XCOLOR(triang_xcolor[0]);
	    if ((item = next_item())) {
	      SET_XPOINT(triang_xpoint[1]);
	      if ((item = next_item())) {
		SET_XCOLOR(triang_xcolor[1]);
		if ((item = next_item())) {
		  SET_XPOINT(triang_xpoint[2]);
		  if ((item = next_item())) {
		    SET_XCOLOR(triang_xcolor[2]);
		    triang_xpoint[3] = triang_xpoint[0];
/*		    XDrawLines(display, window, gc, 
			       triang_xpoint, 4, CoordModeOrigin); */
		    XFillPolygon(display, window, gc, 
				 triang_xpoint, 4, Convex, CoordModeOrigin);
		  }
		}
	      }
	    }
	  }
	  item = next_item();
	  break;

	case PLOT_LOG_text:
	/*    printf(" TXT");*/
	  SET_XPOINT(text_xpoint);
	  string_len = 0;
	  while((item = next_item())){
	    if (item->type != PLOT_LOG_string) break;
	    for (i=0;i<PLOT_LOG_charlen; i++) {
	      if (item->U.c[i] == 0) break;
	      if (string_len < (STRING_LEN-1)) {
		string[string_len++] = item->U.c[i];
	      }
	    }
	  }
	  string[string_len] = 0;  /* add a final NUL */

	  XDrawString(display,window,gc,
		      text_xpoint.x,text_xpoint.y,string,string_len);


	  /* new_item is already done! */
	  break;

	case PLOT_LOG_null:
	    printf(" NUL");
	  break;

	default:
	  printf("PLOT_LOG_flush: unknown item type %d\n",item->type);
	  break;
	}
      }
      if (line_len > 1) {
	XDrawLines(display, window, gc, 
		   line_xpoint, line_len, CoordModeOrigin);
      }
    }
    XFlush(display);
  }

  PLOT_LOG_flushing = 0;

}
